@@ -1,8 +1,9 @@
 import copy
 import os
+import pickle
 import random
 import sys
-from datetime import datetime
+from datetime import datetime, timedelta
 from typing import Any, List, Optional, Sequence, Text, Tuple, Union, overload
 
 import pygame
@@ -11,12 +12,17 @@ from pygame.key import key_code
 from pygame.locals import *
 from xpinyin import Pinyin
 
+from primaryschool.dirs import *
 from primaryschool.locale import _
 from primaryschool.resource import (default_font, default_font_path,
                                     get_default_font, get_font_path)
 from primaryschool.subjects import *
+from primaryschool.subjects._templates_ import GameBase
 from primaryschool.subjects.yuwen.words import cn_ps_c
 
+# primaryschool.subjects.yuwen.g_pinyin_missile
+module_str = __name__
+
 name_t = _('pinyin missile')
 
 difficulties = [
@@ -181,7 +187,7 @@ class WordSurfacesManager():
         self.lang_code = 'zh_CN'
         self.font_path = get_font_path(self.lang_code, show_not_found=True)
         self.font = pygame.font.Font(self.font_path, self.font_size)
-        self.surfaces = self.get_surfaces()
+        self.surfaces = []
 
     def set_font_size(self, size):
         assert isinstance(size, int)
@@ -190,9 +196,14 @@ class WordSurfacesManager():
     def get_font_size(self):
         return self.font_size
 
-    def get_surfaces(self):
+    def set_surfaces(self):
         assert len(self.pm.words) > 0
-        return [WordSurface(self.pm, self, w) for w in self.pm.words]
+        self.surfaces = [WordSurface(self.pm, self, w) for w in self.pm.words]
+
+    def get_surfaces(self):
+        if not self.surfaces:
+            self.set_surfaces()
+        return self.surfaces
 
     def count(self):
         return len(self.surfaces)
@@ -210,6 +221,17 @@ class WordSurfacesManager():
         self.moving_surfaces.append(ws)
         self.frame_counter = 0
 
+    def save(self, _copy):
+        _copy['0x0'] = [s.word for s in self.surfaces]
+        _copy['0x1'] = [(ms.word, ms.dest) for ms in self.moving_surfaces]
+        return _copy
+
+    def load(self, _copy):
+        for w in _copy['0x0']:
+            self.surfaces.append(WordSurface(self.pm, self, w))
+        for w, d in _copy['0x1']:
+            self.moving_surfaces.append(WordSurface(self.pm, self, w, d))
+
     def blit(self):
         if len(self.surfaces) > 0:
             if len(self.moving_surfaces) < 1:
@@ -301,7 +323,7 @@ class InfoSurface():
     def get_datetime_diff_str(self):
         if self.end_time is None:
             self.end_time = self.win.end_time = datetime.now()
-        diff = self.end_time - self.pm.start_time
+        diff = (self.end_time - self.pm.start_time) + self.pm.last_timedelta
         _h, _rem = divmod(diff.seconds, 3600)
         _min, _sec = divmod(_rem, 60)
         return _('Cost: ') + f'{_h}:{_min}:{_sec}'
@@ -384,7 +406,7 @@ class InfoSurface():
 
 
 class WordSurface():
-    def __init__(self, pm, _manager, word):
+    def __init__(self, pm, _manager, word, dest=None):
         self.pm = pm
         self.win = self.pm.win
         self.manager = _manager
@@ -401,7 +423,7 @@ class WordSurface():
 
         self.surface = self.get_surface()
         self.size = self.get_size()
-        self.dest = self.get_random_dest()
+        self.dest = dest if dest else self.get_random_dest()
         self.center = self.get_center()
         self.pinyin = self.get_pinyin()
 
@@ -491,9 +513,9 @@ class WordSurface():
         return _new
 
 
-class PinyinMissile(SubjectGame):
+class PinyinMissile(GameBase):
     def __init__(self, win):
-        super().__init__(win)
+
         self.win = win
 
         # window
@@ -505,13 +527,16 @@ class PinyinMissile(SubjectGame):
         self.running = True
         self.FPS = self.win.FPS
         self.clock = self.win.clock
+        self._load = False
 
+        self.subject = self.win.subject
         self.subject_index = self.win.subject_index
-        self.game_index = self.win.game_index
+        self.subject_game_index = self.win.subject_game_index
         self.difficulty_index = self.win.difficulty_index
 
         self.main_menu = self.win.main_menu
         self.play_menu = self.win.play_menu
+        self.save_menu = self.win.save_menu
         self.surface = self.win.surface
 
         self._input = ''
@@ -524,7 +549,6 @@ class PinyinMissile(SubjectGame):
         self.word = Word(self)
         self.words = self.word.get_words(self.difficulty_index)
         self.wordsurfaces_manager = WordSurfacesManager(self)
-        self.word_surfaces = self.wordsurfaces_manager.get_surfaces()
 
         self.wave = Wave(self)
 
@@ -532,17 +556,16 @@ class PinyinMissile(SubjectGame):
         self.lose_count = 0
         self.word_count = len(self.words)
 
+        self.copy_path = get_copy_path(module_str)
+
         self.print_game_info()
 
+        self.last_timedelta = timedelta(0)
         self.start_time = datetime.now()
         self.end_time = None
 
     def print_game_info(self):
-        print(
-            self.win.subjects[self.subject_index].name_t,
-            self.win.games[self.game_index].name_t,
-            difficulties[self.difficulty_index]
-        )
+        print(self.subject.name_t, name_t, difficulties[self.difficulty_index])
 
     def ascii_not_symbol(self, code):
         return 48 <= code <= 57 or 65 <= code <= 90 or 97 <= code <= 122
@@ -553,8 +576,10 @@ class PinyinMissile(SubjectGame):
                 exit()
             elif e.type == pygame.KEYDOWN:
                 if e.key == pygame.K_ESCAPE:
-                    self.play_menu._menu.enable()
-                    self.running = not self.running
+                    self.save_menu._menu.enable()
+                    self.last_timedelta += datetime.now() - self.start_time
+                    self.save_menu._menu.mainloop(self.surface)
+                    self.start_time = datetime.now()
                     return
                 elif e.key == pygame.K_BACKSPACE:
                     self._input = self._input[0:-1]
@@ -565,10 +590,44 @@ class PinyinMissile(SubjectGame):
                     self.input_surface._update()
                     return
 
-    def run(self):
-        
-        self.play_menu._menu.disable()
-        self.play_menu._menu.full_reset()
+    def load(self):
+        try:
+            self._load = True
+            with open(self.copy_path, 'rb') as f:
+                _copy = pickle.load(f)
+            self.wordsurfaces_manager.load(_copy)
+            self.word_count, self.win_count, self.lose_count = _copy['0x2']
+            self.last_timedelta = _copy['0x3']
+            self.start()
+        except e:
+            print(e)
+
+    def save(self):
+        _copy = {}
+        self.wordsurfaces_manager.save(_copy)
+        _copy['0x2'] = (self.word_count, self.win_count, self.lose_count)
+        _copy['0x3'] = (datetime.now() - self.start_time) + self.last_timedelta
+
+        # https://docs.python.org/3/library/pickle.html?highlight=pickle
+        # Warning:
+        # The pickle module is not secure. Only unpickle data you trust.
+        with open(self.copy_path, 'wb') as f:
+            pickle.dump(_copy, f)
+
+    def _start(self):
+
+        if not self._load:
+            self.wordsurfaces_manager.set_surfaces()
+
+    def play(self):
+        self._load = False
+        self.wordsurfaces_manager.surfaces = []
+        self.wordsurfaces_manager.set_surfaces()
+        self.start()
+
+    def start(self):
+
+        self._start()
 
         while self.running:
             self.clock.tick(self.FPS)
@@ -594,6 +653,5 @@ class PinyinMissile(SubjectGame):
             pygame.display.update()
 
 
-def play(win):
-    PinyinMissile(win).run()
-    pass
+def enjoy(win):
+    return PinyinMissile(win)
